@@ -4,9 +4,9 @@ use strict;
 use Carp;
 use SVG::TT::Graph;
 use base qw(SVG::TT::Graph);
-use vars qw($VERSION $TEMPLATE_FH);
-$VERSION = $SVG::TT::Graph::VERSION;
-$TEMPLATE_FH = \*DATA;
+
+our $VERSION = $SVG::TT::Graph::VERSION;
+our $TEMPLATE_FH = \*DATA;
 
 use Data::Dumper;
 use HTTP::Date;
@@ -188,6 +188,9 @@ Whether or not to tidy the content of the SVG file (XML::Tidy required).
 Set the path to an external stylesheet, set to '' if
 you want to revert back to using the default internal version.
 
+Set to "inline:<style>...</style>" with your CSS in between the tags.
+You can thus override the default style without requireing an external URL.
+
 The default stylesheet handles up to 12 data sets. All data series over
 the 12th will have no style and be in black. If you have over 12 data
 sets you can assign them all random colors (see the random_color()
@@ -261,6 +264,8 @@ to 1, set to '0' if you want to turn them off.
 
 Format string for presenting the X axis labels.
 The POSIX strftime() function is used for formatting
+after calling the POSIX tzset() function with the timezone
+specified in timescale_time_zone if present 
 (see strftime man pages and LC_TIME locale information).
 
 =item show_y_labels()
@@ -282,6 +287,15 @@ The expected form is:
 The default time period if not provided is 'days'.
 These time periods are used by the L<DateTime::Duration> methods.
 
+=item timescale_time_zone
+
+This determines the time zone used for the date intervals on the X axis.
+Values are those that L<DateTime> accepts for its constructor's 'time_zone'
+parameter. The default is 'floating'.  If passing in data for a different
+timezone than that set for your system, note that you also must embed the 
+timezone information into the values passed to 'add_data', for example by 
+formatting your DateTime objects with L<DateTime::Format::RFC3339>. 
+
 =item stagger_x_labels()
 
 This puts the labels at alternative levels so if they
@@ -417,6 +431,7 @@ sub _set_defaults {
     'show_y_labels'       => 1,
     'scale_divisions'     => '',
     'min_scale_value'     => '0',
+    'max_scale_value'     => '',
 
     'stacked'             => 0,
 
@@ -424,6 +439,7 @@ sub _set_defaults {
     'stagger_x_labels'    => 0,
     'rotate_x_labels'     => 0,
     'timescale_divisions' => '',
+    'timescale_time_zone' => 'floating',
     'x_label_format'      => '%Y-%m-%d %H:%M:%S',
     'x_label_formatter'   => sub { return @_ },
     'y_label_formatter'   => sub { return @_ },
@@ -529,11 +545,29 @@ sub add_data {
   return 1;
 }
 
+#This is for internal use only, so we don't have to pass the time zone around.
+#It gets local()ed below before it's used.
+our $_TIME_ZONE;
+
+sub burn {
+  my ($self, @args) = @_;
+
+  local $_TIME_ZONE = $self->{'config'}{'timescale_time_zone'};
+
+  return $self->SUPER::burn(@args);
+}
+
 # Helper function for doing date/time calculations
 # Current implementations of DateTime can be slow :-(
 sub dateadd {
   my ($epoch,$value,$unit) = @_;
   my $dt = DateTime->from_epoch(epoch => $epoch);
+
+  #This should have been local()ed prior to getting here.
+  if ( $_TIME_ZONE ) {
+    $dt->set_time_zone( $_TIME_ZONE );
+  }
+
   $dt->add( $unit => $value );
   return $dt->epoch();
 }
@@ -542,6 +576,11 @@ sub dateadd {
 sub calculations {
   my $self = shift;
 
+  # Need to set the timezone in order for x-axis labels to be 
+  # formatted correctly by strftime:
+  $ENV{'TZ'} = $self->{config}->{timescale_time_zone};
+  POSIX::tzset() if ($] lt '5.008009');
+
   # run through the data and calculate maximum and minimum values
   my ($max_key_size,$max_time,$min_time,$max_value,$min_value,$max_x_label_length,$x_label);
 
@@ -612,8 +651,11 @@ __DATA__
 [% USE date %]
 [% stylesheet = 'included' %]
 
-[% IF config.style_sheet && config.style_sheet != '' %]
+[% IF config.style_sheet && config.style_sheet != '' && config.style_sheet.substr(0,7) != 'inline:' %]
   <?xml-stylesheet href="[% config.style_sheet %]" type="text/css"?>
+[% ELSIF config.style_sheet && config.style_sheet.substr(0,7) == 'inline:'%]
+  [% stylesheet = 'inline'
+     style_inline = config.style_sheet.substr(7) %]
 [% ELSE %]
   [% stylesheet = 'excluded' %]
 [% END %]
@@ -625,7 +667,9 @@ __DATA__
 <!-- Dave Meibusch                 -->
 <!-- ////////////////////////////  -->
 
-[% IF stylesheet == 'excluded' %]
+[% IF stylesheet == 'inline' %]
+[% style_inline %]
+[% ELSIF stylesheet == 'excluded' %]
 [%# include default stylesheet if none specified %]
 <defs>
 <style type="text/css">
@@ -898,8 +942,8 @@ __DATA__
 </clipPath>
 
 <!-- axis -->
-<path d="M[% x %] [% y %] v[% h %]" class="axis" id="xAxis"/>
-<path d="M[% x %] [% base_line %] h[% w %]" class="axis" id="yAxis"/>
+<path d="M[% x %] [% base_line %] h[% w %]" class="axis" id="xAxis"/>
+<path d="M[% x %] [% y %] v[% h %]" class="axis" id="yAxis"/>
 
 <!-- //////////////////////////////  AXIS DISTRIBUTIONS //////////////////////////// -->
 <!-- x axis scaling -->
